﻿using System;
using System.Threading.Tasks;
using ELTE.TicTacToeGame.Model;
using ELTE.TicTacToeGame.Persistence;
using ELTE.TicTacToeGame.View;
using ELTE.TicTacToeGame.ViewModel;
using Xamarin.Forms;

namespace ELTE.TicTacToeGame
{
    public class App : Application
    {
        #region Private fields

        private IPersistence _persistence;
        private TicTacToeModel _model;
        private TicTacToeViewModel _viewModel;

        #endregion

        #region Constructors

        /// <summary>
        /// Alkalmazás példányosítása.
        /// </summary>
        public App()
        {
            _persistence = DependencyService.Get<IPersistence>(); // a perzisztencia betöltése az adott platformon

            _model = new TicTacToeModel(_persistence);
            _model.GameWon += new EventHandler<GameWonEventArgs>(Model_GameWon);
            _model.GameOver += new EventHandler(Model_GameOver);

            _viewModel = new TicTacToeViewModel(_model);
            _viewModel.LoadGame += new EventHandler(ViewModel_LoadGame); // kezeljük a nézetmodell eseményeit
            _viewModel.SaveGame += new EventHandler(ViewModel_SaveGame);

            MainPage = new MainPage();
            MainPage.BindingContext = _viewModel;
        }

        #endregion

        #region Application event methods

        protected override void OnStart()
        {
            _model.NewGame(); // indításkor új játékot kezdünk

            // betöltjük a felfüggesztett játékot, amennyiben van
            try
            {
                Task.Run(async () => await _model.LoadGameAsync("SuspendedGame"));
            }
            catch { }
        }

        protected override void OnSleep()
        {
            // elmentjük a jelenleg folyó játékot
            try
            {
                Task.Run(async () => await _model.SaveGameAsync("SuspendedGame"));
                // az aszinkron tevékenységet taszkban futtatjuk
            }
            catch { }
        }

        protected override void OnResume()
        {
            // betöltjük a felfüggesztett játékot, amennyiben van
            try
            {
                Task.Run(async () => await _model.LoadGameAsync("SuspendedGame"));
            }
            catch { }
        }

        #endregion

        #region Model event handlers

        /// <summary>
        /// Játék megnyerésének eseménykezelése.
        /// </summary>
        private async void Model_GameWon(object sender, GameWonEventArgs e)
        {
            switch (e.Player)
            {
                case Player.PlayerO:
                    await MainPage.DisplayAlert("Játék vége!", "A kör játékos győzött!", "OK");
                    break;
                case Player.PlayerX:
                    await MainPage.DisplayAlert("Játék vége!", "A kereszt játékos győzött!", "OK");
                    break;
            }
        }

        /// <summary>
        /// Játék végének eseménykezelése.
        /// </summary>
        private async void Model_GameOver(object sender, EventArgs e)
        {
            await MainPage.DisplayAlert("Döntetlen játék!", "Játék vége!", "OK");
        }

        #endregion

        #region ViewModel event handlers

        /// <summary>
        /// Játék betöltésének eseménykezelője.
        /// </summary>
        private async void ViewModel_LoadGame(object sender, System.EventArgs e)
        {
            try
            {
                await _model.LoadGameAsync("LastGame");
            }
            catch { }
        }

        /// <summary>
        /// Játék mentésének eseménykezelője.
        /// </summary>
        private async void ViewModel_SaveGame(object sender, System.EventArgs e)
        {
            try
            {
                await _model.SaveGameAsync("LastGame");
            }
            catch
            {
                await MainPage.DisplayAlert("Hiba!", "A mentés sikertelen volt.", "OK");
            }
        }

        #endregion
    }
}
